/*
 * An object for a directory provided by an Ext2-compatible filesystem.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <ext2fs/ext2fs.h>

#include <fsclient/file.h>
#include <fsserver/ext2_file_operations.h>
#include <fsserver/directory_resource.h>



/* Forward declarations. */

struct Ext2DirectoryState;



/* Support for providing access to files. */

class Ext2Directory : public DirectoryResource
{
protected:
    Ext2FileOperations *_ops;
    fileid_t _fileid;

    virtual long _opendir(flags_t flags, offset_t *size, l4_cap_idx_t *cap,
                          object_flags_t *object_flags);

public:
    explicit Ext2Directory(Ext2FileOperations *ops, fileid_t fileid)
    : _ops(ops), _fileid(fileid)
    {
    }

    virtual ~Ext2Directory();

    /* Helper methods. */

    void read_directory(fileid_t fileid, file_t *writer);

    int write_directory_entry(struct Ext2DirectoryState *dir);
};



/* Helper structures. */

struct Ext2DirectoryState
{
    Ext2Directory *directory;
    file_t *writer;
    struct ext2_dir_entry *entry;
    long offset;
};

// vim: tabstop=4 expandtab shiftwidth=4
